Type: Package
Package: geocodebr
Title: Geolocalização De Endereços Brasileiros (Geocoding Brazilian
        Addresses)
Version: 0.6.3
Authors@R: c(
    person("Rafael H. M.", "Pereira", , "rafa.pereira.br@gmail.com", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0003-2125-7465")),
    person("Daniel", "Herszenhut", , "dhersz@gmail.com", role = "aut",
           comment = c(ORCID = "0000-0001-8066-1105")),
    person("Gabriel", "Garcia de Almeida", role = "aut",
           comment = c(ORCID = "0009-0003-3557-7328")),
    person("Arthur", "Bazolli", , "baz.arthur@gmail.com", role = "ctb"),
    person("Pedro", "Milreu Cunha", , "pedro.cunha@ipea.gov.br", role = "ctb"),
    person("ITpS - Instituto Todos pela Saúde", role = "fnd"),
    person("Ipea - Instituto de Pesquisa Econômica Aplicada", role = c("cph", "fnd"))
  )
Description: Método simples e eficiente de geolocalizar dados no Brasil. O 
    pacote é baseado em conjuntos de dados espaciais abertos de endereços 
    brasileiros, utilizando como fonte principal o Cadastro Nacional de Endereços 
    para Fins Estatísticos (CNEFE). O CNEFE é publicado pelo Instituto Brasileiro 
    de Geografia e Estatística (IBGE), órgão oficial de estatísticas e geografia 
    do Brasil. (A simple and efficient method for geolocating data in Brazil. The 
    package is based on open spatial datasets of Brazilian addresses, primarily 
    using the Cadastro Nacional de Endereços para Fins Estatísticos (CNEFE), 
    published by the Instituto Brasileiro de Geografia e Estatística (IBGE), 
    Brazil's official statistics and geography agency.)
License: MIT + file LICENSE
URL: https://github.com/ipea/geocodebr,
        https://ipea.github.io/geocodebr/
BugReports: https://github.com/ipea/geocodebr/issues
Depends: R (>= 4.1.0)
Imports: arrow (>= 15.0.1), checkmate, callr, cli, data.table, DBI,
        dplyr, duckdb, duckspatial (>= 1.0.0), enderecobr (>= 0.5.0),
        fs, glue, h3r, httr2 (>= 1.0.0), nanoarrow (>= 0.3.0.1),
        parallelly, purrr, rlang, sf, sfheaders, tools
Suggests: covr, dbplyr, geobr, ggplot2 (>= 3.3.1), knitr, rmarkdown,
        scales, testthat (>= 3.0.0)
VignetteBuilder: knitr
Config/testthat/edition: 3
Encoding: UTF-8
Language: pt
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-05-24 00:41:18 UTC; rafap
Author: Rafael H. M. Pereira [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-2125-7465>),
  Daniel Herszenhut [aut] (ORCID:
    <https://orcid.org/0000-0001-8066-1105>),
  Gabriel Garcia de Almeida [aut] (ORCID:
    <https://orcid.org/0009-0003-3557-7328>),
  Arthur Bazolli [ctb],
  Pedro Milreu Cunha [ctb],
  ITpS - Instituto Todos pela Saúde [fnd],
  Ipea - Instituto de Pesquisa Econômica Aplicada [cph, fnd]
Maintainer: Rafael H. M. Pereira <rafa.pereira.br@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-24 14:40:02 UTC
Built: R 4.5.2; ; 2026-05-24 18:27:36 UTC; unix
