# Generated by roxygen2: do not edit by hand

export(busca_por_cep)
export(definir_campos)
export(definir_pasta_cache)
export(deletar_pasta_cache)
export(download_cnefe)
export(geocode)
export(geocode_reverso)
export(listar_dados_cache)
export(listar_pasta_cache)
importFrom(data.table,"%chin%")
importFrom(data.table,"%like%")
importFrom(data.table,":=")
importFrom(data.table,.GRP)
importFrom(data.table,.I)
importFrom(data.table,.N)
importFrom(data.table,.SD)
importFrom(dplyr,across)
importFrom(dplyr,all_of)
importFrom(dplyr,case_when)
importFrom(dplyr,mutate)
importFrom(dplyr,select)
importFrom(stats,na.omit)
importFrom(stats,weighted.mean)
