#' @param h3_res Um número que indica a resolução espacial da célula hexagonal
#'    H3 da localização dos pontos retornados. Também aceita um vetor de números,
#'    e.g. `c(8, 9)` Por padrão, é `NULL`. Detalhes sobre as resoluções disponíveis
#'    em \url{https://h3geo.org/docs/core-library/restable/}
