#' @param cache Um valor lógico. Indica se os dados do CNEFE devem ser salvos ou
#'   lidos do cache, reduzindo o tempo de processamento em chamadas futuras. O
#'   padrão é `TRUE`. Quando `FALSE`, os dados do CNEFE são baixados para um
#'   diretório temporário.
