#' @param n_cores Um número. O número de núcleos de CPU a serem utilizados no
#'   processamento dos dados. Por padrão, `n_cores = NULL` e o pacote utiliza o
#'   número máximo de cores físicos disponíveis.
