# errors if could not download one or more files

    Code
      tester(cache = FALSE)
    Condition <geocodebr_error_cnefe_download_failed>
      Error in `download_cnefe()`:
      ! Could not download one or more CNEFE data files.
      i Please try again later.

