# errors when all fields are NULL

    Code
      tester()
    Condition <geocodebr_error_null_address_fields>
      Error in `definir_campos()`:
      ! Pelo menos um campo não pode ser nulo `NULL`.

