% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cache.R
\name{definir_pasta_cache}
\alias{definir_pasta_cache}
\title{Define um diretório de cache para o geocodebr}
\usage{
definir_pasta_cache(path, verboso = TRUE)
}
\arguments{
\item{path}{Uma string. O caminho para o diretório usado para armazenar os
dados em cache.  Se \code{NULL}, o pacote usará um diretório versionado salvo
dentro do diretório retornado por \code{\link[tools:userdir]{tools::R_user_dir()}}.}

\item{verboso}{Um valor lógico. Indica se barras de progresso e mensagens
devem ser exibidas durante o download dos dados do CNEFE e a geocodificação
dos endereços. O padrão é \code{TRUE}.}
}
\value{
Retorna de forma invisível o caminho do diretório de cache.
}
\description{
Define um diretório de cache para os dados do geocodebr. Essa configuração
é persistente entre sessões do R.
}
\examples{
definir_pasta_cache(tempdir())

# retoma pasta padrão do pacote
definir_pasta_cache( path = NULL)

}
