% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cache.R
\name{apaga_data_release_antigo}
\alias{apaga_data_release_antigo}
\title{Atualiza dados no release local}
\usage{
apaga_data_release_antigo()
}
\value{
Retorna de forma invisível o caminho do diretório de cache.
}
\description{
Detecta automaticamente se o release local esta desatualizado. Se sim, apaga
a pasta de cache do release local e atualiza a versao do release no caminho
da pasta
}
\keyword{internal}
