% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{add_precision_col}
\alias{add_precision_col}
\title{Add a column with info of geocode match_type}
\usage{
add_precision_col(con, update_tb = NULL)
}
\arguments{
\item{con}{A db connection}

\item{update_tb}{String. Name of a table to be updated in con}
}
\value{
Adds a new column to a table in con
}
\description{
Add a column with info of geocode match_type
}
\keyword{internal}
