% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geocodebr-package.R
\docType{package}
\name{geocodebr}
\alias{geocodebr}
\alias{geocodebr-package}
\title{Package: geocodebr: Geolocalização De Endereços Brasileiros (Geocoding
Brazilian Addresses)}
\description{
\if{html}{\figure{logo.png}{options: style='float: right' alt='logo' width='120'}}

Método simples e eficiente de geolocalizar dados no Brasil. O pacote é baseado em conjuntos de dados espaciais abertos de endereços brasileiros, utilizando como fonte principal o Cadastro Nacional de Endereços para Fins Estatísticos (CNEFE). O CNEFE é publicado pelo Instituto Brasileiro de Geografia e Estatística (IBGE), órgão oficial de estatísticas e geografia do Brasil. (A simple and efficient method for geolocating data in Brazil. The package is based on open spatial datasets of Brazilian addresses, primarily using the Cadastro Nacional de Endereços para Fins Estatísticos (CNEFE), published by the Instituto Brasileiro de Geografia e Estatística (IBGE), Brazil's official statistics and geography agency.)
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/ipeaGIT/geocodebr}
  \item \url{https://ipeagit.github.io/geocodebr/}
  \item Report bugs at \url{https://github.com/ipeaGIT/geocodebr/issues}
}

}
\author{
\strong{Maintainer}: Rafael H. M. Pereira \email{rafa.pereira.br@gmail.com} (\href{https://orcid.org/0000-0003-2125-7465}{ORCID})

Authors:
\itemize{
  \item Daniel Herszenhut \email{dhersz@gmail.com} (\href{https://orcid.org/0000-0001-8066-1105}{ORCID})
  \item Gabriel Garcia de Almeida (\href{https://orcid.org/0009-0003-3557-7328}{ORCID})
}

Other contributors:
\itemize{
  \item Arthur Bazolli \email{baz.arthur@gmail.com} [contributor]
  \item Pedro Milreu Cunha \email{pedro.cunha@ipea.gov.br} [contributor]
  \item ITpS - Instituto Todos pela Saúde [funder]
  \item Ipea - Instituto de Pesquisa Econômica Aplicada [copyright holder, funder]
}

}
\keyword{internal}
