% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cache.R
\name{deletar_pasta_cache}
\alias{deletar_pasta_cache}
\title{Deletar pasta de cache do geocodebr}
\usage{
deletar_pasta_cache()
}
\value{
Retorna de forma invisível o caminho do diretório de cache.
}
\description{
Deleta todos arquivos da pasta do cache.
}
\examples{
\dontshow{if (identical(TRUE, FALSE)) withAutoprint(\{ # examplesIf}
deletar_pasta_cache()
\dontshow{\}) # examplesIf}
}
