% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{cache_message}
\alias{cache_message}
\title{Message when caching file}
\usage{
cache_message(
  local_file = parent.frame()$local_file,
  cache = parent.frame()$cache
)
}
\arguments{
\item{local_file}{The address of a file passed from the download_file function.}

\item{cache}{Logical. Whether the cached data should be used.}
}
\value{
A message
}
\description{
Message when caching file
}
\keyword{internal}
