% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{create_index}
\alias{create_index}
\title{create index}
\usage{
create_index(con, tb, cols, operation, overwrite = TRUE)
}
\description{
create index
}
\keyword{internal}
