% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cache.R
\name{listar_dados_cache}
\alias{listar_dados_cache}
\title{Listar dados em cache}
\usage{
listar_dados_cache(print_tree = FALSE)
}
\arguments{
\item{print_tree}{Um valor lógico. Indica se o conteúdo da pasta de cache
deve ser exibido em um formato de árvore. O padrão é \code{FALSE}.}
}
\value{
O caminho para os arquivos em cache
}
\description{
Lista os dados salvos localmente na pasta de cache
}
\examples{
listar_dados_cache()

listar_dados_cache(print_tree = TRUE)

}
