% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cache.R
\name{listar_pasta_cache}
\alias{listar_pasta_cache}
\title{Obtém a pasta de cache usado no geocodebr}
\usage{
listar_pasta_cache()
}
\value{
O caminho da pasta de cache.
}
\description{
Obtém o caminho da pasta utilizada para armazenar em cache os dados do
geocodebr. Útil para inspecionar a pasta configurada com \code{\link[=definir_pasta_cache]{definir_pasta_cache()}}
em uma sessão anterior do R. Retorna a pasta de cache padrão caso nenhuma
pasta personalizado tenha sido configurada anteriormente.
}
\examples{
listar_pasta_cache()

}
